//+------------------------------------------------------------------+
//|                                                    Exception.mq4 |
//|                                 Copyright  2011-2016, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//| MQL's OOP notes: rubber arrays, safe numbers,                    |
//|                             exceptions handling and other tricks |
//|                        https://www.mql5.com/en/blogs/post/683241 |
//|                        https://www.mql5.com/en/blogs/post/683243 |
//+------------------------------------------------------------------+

#property strict

#include <Exception.mqh>
#include <SafeNumbers.mqh>
#include <SafeArray.mqh>


void subfunc()
{
  Number<double> d(10.5);
  Number<double> x;
  Number<double> y;
  x = 0;
  
  TRY
  {
    y = d / x;        // no error here, and y is DBL_MAX  
    Print(x.toString(), ", ", y.toString());
  }
  CATCH(e)
  {
    Print(e.location, " : ", e.message);
    e.throw(DBL_MAX); // pass DBL_MAX as result of division by 0
  }
  END
}

void subfunc2()
{
  TRY
  {
    Number<int> x(0);
    Number<int> y(1);
    y /= x;
    Print(y.toString());
  }
  CATCH(e)
  {
    Print(e.location, " : ", e.message);
  }
  END
}

void OnStart()
{
  TRY
  {
    subfunc();

    RubbArray<double> a;
    TRY                // nested try-blocks are allowed
    {
      a << 1 << 2;
      a[edit(3)] = 10; // assignment has no effect, there is no fatal error
    }
    CATCH(e2)
    {
      Print(e2.location, " : ", e2.message);
    }
    END
    
    Print(a[3]);     // will print 15
    
    subfunc2();
  }
  CATCH(e1)
  {
    Print(e1.location, " : ", e1.message);
    e1.throw(15); // pass 15 as a result of expression which caused the error
  }
  END
}
